#include <stdio.h>
#include <stdlib.h>
#include <stream.h>
#include <math.h>
#include <time.h>
#include "magm.h"
#include <fstream.h>
#include <sys/timeb.h>
#include <time.h>
#define NG 2000000000


Representacion::Representacion(int b,int ngenes,double *linf, double *lsup, int *pres,int iguales){
  int j=0;
  bits=0;
  base=b;  
  presicion=new(int[ngenes]);
  bitsXgene=new(int[ngenes]);
  lInf=new(double[ngenes]);
  lSup=new(double[ngenes]);
  for(int i=0;i<ngenes;i++){
    presicion[i]=pres[j];//cuantos decimales de presicion se trabajara
    //calculo del numero de bits
    bitsXgene[i]=(int)ceil(log((lsup[j]-linf[j])*pow(10,pres[j])*1.000001)/log(base));
    bits+=bitsXgene[j];
    lInf[i]=linf[j];//lower bound
    lSup[i]=lsup[j];//upperbound
    if(iguales!=1)j++;
  }
  cout<<bits<<"bits\n";
  nGenes=ngenes;
}
int Representacion::retBits(){
  return bits;
}

Representacion::~Representacion(){
  delete [] presicion;
  delete [] bitsXgene;
  delete [] lInf;
  delete [] lSup;
}

/*
 Metodo que copia un arreglo de enteros (genotipo) hacia memoria y regresa un puntero hacia ese espacio de memoria 
*/

void Representacion::copia(int *num,int *num2,int n)
{
  int *pnum=num;
  for(int i=0;i<n;i++,pnum++,num2++)//recorre todo el genotipo,haciendo
    *pnum=*num2;//una copia hacia la memoria creada.
}

void Representacion::copia(double *num,double *num2,int n)
{
  double *pnum=num;
  for(int i=0;i<n;i++,pnum++,num2++)//recorre todo el genotipo,haciendo
    *pnum=*num2;//una copia hacia la memoria creada.
}

/*
Metodo que convierte de Gray a binario
*/
int *Representacion::gray_A_binario(int *g){
  int *b=new int[bits];
  int *pb;
  pb=b;
  int valor=*g;
  *pb=valor;
  g++;
  pb++;
  for(int k=1;k<bits;k++,g++,pb++){
    if(*g==1)valor=(valor==1)?0:1; //simula el not.
    *pb=valor;
  }
  return b;
}

/*
  metodo que a partir de un arreglo de enteros (genotipo(simulacion de arreglo binario)) de codifica su valor (fenotipo) dentro de un rango minimo y un rango maximo 
*/
double Representacion::decodifica (int cual,int *num)
{
  double fenotipo=0.0;
  int cantbits=0;
  int i;
  int *cadena,*pb;
  if(base==2) //si es base 2 entonces  decodifica a binario
    cadena=gray_A_binario(num);
  else cadena=num;
  for(i=0;i<cual;i++)
    cantbits+=bitsXgene[i];
  
  pb=cadena;
  for(i=0;i<bitsXgene[cual];i++)//recorrer todo el genotipo, para hacer la suma
    fenotipo+=ldexp(pb[i+cantbits],i);//de las potencias de dos reespectivas y
  if(base==2)
    delete [] cadena;
  double regreso=(lInf[cual]+fenotipo*(lSup[cual]-lInf[cual])/(pow(base,bitsXgene[cual])-1));  //mediante la formula que dio en clase adecuo los valores dentro de los 
  return regreso;
}

/*
metodo que imprime el genotipo
*/
void Representacion::imprime(double *num,int n)
{
  for(int i=0;i<n;i++,num++)//recorre el genotipo, e imprime su valor.
    cout<<*num<<" ";
}

void Representacion::imprime(int *num,int n){
  for(int i=0;i<n;i++,num++)//recorre el genotipo, e imprime su valor.
    cout<<*num<<" ";
}

/*
  metodo que genera un genotipo con la seleccion del valor de los alelos al azar con probabilidad de 0.5
*/
int *Representacion::genera(){
  int *gtemp,*genotipo;
  genotipo=new(int[bits]);//asigna memoria hacia el genotipo a generar
  gtemp=genotipo;
  for(int i=0;i<bits;i++,gtemp++)//genera el genotipo, alelo a alelo,mediante
    *gtemp=rnd(0,base-1);//probabilidad.
  return genotipo;//regresa el genotipo generado
}

/*
  metodo que genera un genotipo con la seleccion del valor de los alelos al azar con probabilidad de 0.5
*/
int *Representacion::inicializa(){
  int *gtemp,*genotipo;
  genotipo=new(int[bits]);//asigna memoria hacia el genotipo a generar
  gtemp=genotipo;
  for(int i=0;i<bits;i++,gtemp++)//genera el genotipo, alelo a alelo,mediante
    *gtemp=0;//probabilidad.
  return genotipo;//regresa el genotipo generado
}


/*
  Metodo que cruza dos genotipos, a partir de dos posiciones dadas
  (cruza de dos puntos)
*/
void Representacion::cruza(int *padre1,int *padre2,int posicion1,int posicion2){
  int cambio;
  for(int i=0;i<bits;i++,padre1++,padre2++){
    cambio=(posicion1<=i&&i<=posicion2)?1:0;
    if(cambio){
      int t=*padre1;
      *padre1=*padre2;
      *padre2=t;
    }
  }
}

/*
  metodo que muta a un genotipo a partir de una probabilidad dada y regresa el numero de mutaciones efecutadas
*/
int Representacion::muta(int *padre,float Pm){
  int *ptemp, mutaciones=0,r;
  ptemp=padre;
  for(int i=0;i<bits;i++,ptemp++)//recorre el genotipo, y si el flip da 
    if(flip(Pm)==1){//1 entonces muta
      mutaciones++;
      do
	r=rnd(0,base-1);
      while(r==*ptemp);
      *ptemp=r;
    }
  return mutaciones;//regresa el total de las mutaciones.
}

 
MicroAG::MicroAG(int itot, int gmax,int m,double pc,double pm,int ngenes,int nobjs, int tampareto,int tammem, int ndivs,int b,double *linf,double *lsup,int *pres,int iguales,int sndelit,double pNRem,char *arch):Representacion(b,ngenes,linf,lsup,pres,iguales)//,Funcion(nobjs,ngenes,ale)
{
  sndElit=sndelit;
  nDivs=ndivs;
  tamPARETO=tampareto;
  Pc=pc;
  Pm=pm;
  M=m;
  GMax=gmax;
  iTot=itot;
  nObjs=nobjs;
  cantMem=2;
  cicloDeReemplazo=20;
  tamDiv=new(double[nObjs]);
  inicioMalla=new(double[nObjs]);
  int i=nObjs-1;
  lMalla=(long int)floor(pow(nDivs,nObjs));
  hipercubo=new(int[lMalla]);
  pPobNRem=pNRem;
  tamMem=tammem;
  pobSRem=(int)floor(tamMem*(1-pPobNRem));
  iniciar(arch);
}
void MicroAG::iniciar(char *arch){    
  archivo.open(arch,ios::out); 
  srand(time(0)); 
  initrandom(time(0)); 
  Rseed=randreal2(); 
  randomize();
  numCruzas=0;
  numMutaciones=0;
  posMemoria=0;
  posPARETO=0;
}
MicroAG::~MicroAG(){
  
 int i;  
  for(i=0;i<M;i++){
    delete [] poblacion[i].aptObj;
    delete [] poblacion[i].genotipo;
    delete [] poblacionNueva[i].aptObj;
    delete [] poblacionNueva[i].genotipo;
  }
  
  delete [] poblacion;
  delete [] poblacionNueva;
  
  for(i=0;i<tamPARETO;i++){
    delete [] PARETO[i].genotipo;
    delete [] PARETO[i].aptObj;
  }
  for(i=0;i<tamMem;i++){
    delete [] memoria[i].genotipo;
    delete [] memoria[i].aptObj;
  } 
  delete [] memoria;
  delete [] PARETO;
  
  delete [] hipercubo;
  delete [] tamDiv;
  delete [] inicioMalla;
  archivo.close();
}
void MicroAG::almacena(double *num, int n){
  for(int i=0;i<n;i++,num++)//recorre el genotipo, e imprime su valor.
    archivo<<*num<<" ";
}


/*
  Metodo que genera una poblacion aleatoria inicial.
 */
void MicroAG::generaPoblacion(){
  poblacion=new (Individuo[M]);
  poblacionNueva=new (Individuo[M]);
  for(int i=0;i<M;i++){
    poblacion[i].genotipo=genera();
    poblacionNueva[i].genotipo=genera();
    poblacion[i].aptObj=new (double[nObjs]);
    poblacionNueva[i].aptObj=new (double[nObjs]);
    poblacion[i].loc=0;
    poblacionNueva[i].loc=0;
    evalua(poblacion[i].genotipo,poblacion[i].aptObj);
  }

  iniciaMemoria();
}

/*
Funcion que genera la poblacion de la memoria la cual tendra la labor de dar diversidad a la poblacion
 */
void MicroAG::iniciaMemoria(){
  int i=0;
  memoria=new(Individuo[(int)tamMem]);
  PARETO=new(Individuo[tamPARETO]);
  
  for(i=0;i<tamPARETO;i++){
    PARETO[i].genotipo=inicializa();
    PARETO[i].aptObj=new (double[nObjs]);
    PARETO[i].loc=0;
  }
  for(i=0;i<tamMem;i++){
    memoria[i].genotipo=genera();
    memoria[i].aptObj=new(double[nObjs]);
    memoria[i].loc=0;
    evalua(memoria[i].genotipo,memoria[i].aptObj);
    FunPARETO(memoria+i);
  }
}

/*
  Metodo donde se efectua la cruza, y la mutacion.

 */
void MicroAG::cruzaYMutacion(){
  //se recorre la poblacion generada, de 2 en dos, y de acuerdo al Pc y a Pm 
  //se cruzan y se mutan
  int pDeCruza[2],menor;
  for(int i=0;i<M;i+=2){
    if(flip(Pc)){//para checar si va a haber cruza
      numCruzas++;
      pDeCruza[0]=(int)rnd(1,bits-2);//se busca el punto de 
      //cruza1 y dos  y se realiza la cruza
      do
	pDeCruza[1]=(int)rnd(1,bits-2);
      while(pDeCruza[1]==pDeCruza[0]);
      menor=(pDeCruza[0]<pDeCruza[1])?0:1;
      cruza(poblacionNueva[i].genotipo,poblacionNueva[i+1].genotipo,pDeCruza[menor],pDeCruza[1-menor]);
    }
    numMutaciones+=muta(poblacionNueva[i].genotipo,Pm);//se muta de acuerdo a Pm
    numMutaciones+=muta(poblacionNueva[i].genotipo,Pm);//se muta de acuerdo a Pm
    evalua(poblacionNueva[i].genotipo,poblacionNueva[i].aptObj);//se calcula su aptitud
    evalua(poblacionNueva[i+1].genotipo,poblacionNueva[i+1].aptObj);//se calcula su aptitud
  }
}

/*
Seleccion por torneo.
 */
void MicroAG::seleccion(){
  Individuo *pp=poblacion,*ppn=poblacionNueva;
  int ind[M*2],i=0,menor,l=0, *selec[M];
  double *apt[M];
  
  for(int j=0;j<M*2;j++)
    ind[j]=rnd(0,M-1);
  l=0;
  while(l<(M*2)){
    switch(Domina(pp+ind[l],pp+ind[l+1])){
    case 11://nadie domina
      if(flip(0.5))
	copiaxxx(ppn+((int)floor(l/2)),pp+ind[l]);
      else
	copiaxxx(ppn+((int)floor(l/2)),pp+ind[l+1]);
      break;
    case 1://el primero domina al segundo
    case 0:
      copiaxxx(ppn+((int)floor(l/2)),pp+(ind[l]));
      break;
    case -1://el segundo domina al primero
      copiaxxx(ppn+((int)floor(l/2)),pp+(ind[l+1]));
      break;
    }
    l+=2;
  }
}

/*
  Funcion que indica si un individuo domina a otro
si el primero domina al segundo retorna -1
si el segundo domina al primero retoruna 1
si son iguales retorna 0
si ninguno domina retorna 11
*/
int MicroAG::Domina(Individuo *I1,Individuo *I2){
  int anterior = 0;
  int mejor;
  double *ind1=I1->aptObj;
  double *ind2=I2->aptObj;

  int a=restricciones(I1->genotipo);
  int b=restricciones(I2->genotipo);
  if(a<b)//el primero domina
    return 1;
  else if(b<a)//el segundo domina
    return -1;
  //  cout<<"hola\n";
  for(int i=0;i<nObjs;i++,ind1++,ind2++){
    if(*ind1 <*ind2)	mejor = 1;
    else if(*ind2<*ind1)mejor = -1;
    else mejor = 0;
    if(mejor!=anterior&&anterior!=0&&mejor!=0) return(11);
    if(mejor!=0) anterior = mejor;
  }
  return(anterior);
}

/*
Funcion que memoriza una solucion de tal manera que solo introduce en la primera mitad de la memoria de que genrerara la deversidad de la poblacion
 */
void MicroAG::memoriza(Individuo *mt){
  if(Domina((memoria+posMemoria),mt)==0)
    copiaxxx(memoria+posMemoria,mt);
  if(posMemoria==pobSRem-1)
    posMemoria=0;
  else posMemoria++;
  
}

/*
Metodo que memoriza los mejores de la generacion presente, de tal forma que memoriza primero al mejor, y si hay otro en la poblacion , que sea diferente a este, y que el primero no domine a este otro individuo entonces memoriza tambien al segundo
*/

void MicroAG::Mejores(int *mejor1,int *mejor2){
  Individuo *pp=poblacion;
  int ind[M],l=0, *selec[M];
  double *apt[M];
  l=1;
  *mejor2=-1;
  *mejor1=0;
  *mejor1=mejorI(poblacion);
  l=0;
  int bandera=1;
  while((l<M)&&bandera){
    if(*mejor1!=l){
      switch(Domina(pp+*mejor1,pp+l)){
      case 11://nadie domina
      case -1://el segundo domina al primero
	*mejor2=l;
	bandera=0;
	break;
      case 1://el primero domina al segundo
      case 0://son iguales
	break;
      }
    }
    l++;
  }
  /*
    memoriza(pp+mejor);
    if(mejor2!=-1)
    memoriza(pp+mejor2);
  */
}

void MicroAG::memorizarMejores(int mejor1,int mejor2){
  Individuo *pp=poblacion;
  memoriza(pp+mejor1);
  if(mejor2!=-1)
    memoriza(pp+mejor2);
}
/*
Metodo que convierte la generacion nueva en la nueva padre.
 */
void MicroAG::nuevaGeneracion(){
  Individuo *p=poblacion;
  poblacion=poblacionNueva;
  poblacionNueva=p;
}

int MicroAG::mejorI(Individuo *pp){
  int l=1;
  int mejor=0;
  while(l<M){
    if(Domina(pp+mejor,pp+l)==-1){
      mejor=l;
      break;
    }
    l++;
  }
  return mejor;
}

/*
Metodo que hace el elitismo
 */
void MicroAG::elitismo(){
  Individuo *pp=poblacion,*ppn=poblacionNueva;
  int ind[M],i=0,mejor=0,l=0, *selec[M];
  double *apt[M];
  l=1;
  mejor=mejorI(poblacion);
  l=0;
  while(l<M){
    int temp=Domina(pp+mejor,ppn+l);
    if(temp==0)
      break;//si todo esto pasa, entonces se mantiene el mejor anterior
    else if(temp==1){
      copiaxxx(ppn+l,pp+mejor);
      l=M;
      break;
    }
    l++;
  }
}

/*
Metodo que obtiene la poblacion de la memoria para crear diversidad de tal manera que elige a la poblacion de forma aleatoria, ya sea a la poblacion que es generada despues de convergencia nominal, o la otra mitad que fue generada aleatoriamente.
 */
void MicroAG::obtPoblacion(){
  int ind;
  Individuo *pp=poblacion;
  for(int i=0;i<M;i++){
    ind=rnd(0,(tamMem-1));
    copiaxxx(pp+i,memoria+ind);
  }
}

/*
  Metodo principal de magm, el cual contiene al algoritmo descrito en el documento adjunto.
 */
void MicroAG::Alg(){
  int *pm1,*pm2,m1,m2;
  timeb tiempo1,tiempo2;
  double tiempo;
  pm1=&m1;
  pm2=&m2;
  ftime(&tiempo1);
  generaPoblacion();
  for(long iter=0;iter<iTot;iter++){
    obtPoblacion();
    for(int conta=0;conta<GMax;conta++){
      seleccion();
      cruzaYMutacion();
      if(iter<sndElit*2)
	elitismo();
      nuevaGeneracion();
    }
    Mejores(pm1,pm2);
    memorizarMejores(m1,m2);
    guardaPobEnPareto(m1,m2);
    sdoElitismo(iter+1);
  }
  salida(PARETO);////,posPARETO,0,0,1);
  cout<<posPARETO<<"\n";
  ftime(&tiempo2);
  tiempo=difftime(tiempo2.time,tiempo1.time)+(double)(tiempo2.millitm-tiempo1.millitm)/1000.0;
  cout<<"tiempo= "<<tiempo<<" segundos\n";
}


void MicroAG::sdoElitismo(long nIt){
  int r=0;
  int numCel=0;
  if(posPARETO==0)return;
  if((nIt%sndElit)==0){
    if(posPARETO==tamPARETO){
      int j=0;
      int *cell=new(int[lMalla*2]);; 
      int *dondeCcell=new(int[lMalla]);; 
      int numInd=0;
      int *contador=new(int[lMalla*2]);
      for(long i=0;i<lMalla;i++){
	if(hipercubo[i]>0){
	  cell[j]=i; //que celda tiene los elementos
	  cell[j+1]=hipercubo[i]; //cuantos elementos tiene la celda
	  dondeCcell[i]=j;
	  j+=2;
	  numCel++;
	  numInd+=cell[j-1];
	}
	//	dondeCcell[i]=0;
      }
      int cuantosxcelda=(int)floor(pobSRem/numCel);
      for(int i=0;i<numCel*2;i+=2){
	contador[i]=cuantosxcelda;
	if(cell[i+1]>cuantosxcelda) contador[i+1]=1;       
	else contador[i+1]=(int)ceil(cuantosxcelda/cell[i+1]);
      }
      for(int i=0;i<posPARETO;i++){
	for(int j=0;j<contador[dondeCcell[PARETO[i].loc]+1];j++){
	  if(contador[dondeCcell[PARETO[i].loc]]>0){
	    contador[dondeCcell[PARETO[i].loc]]--;
	    copiaxxx(memoria+r,PARETO+i);
	    r++;
	  }
 	  else break;  
	}
      }
      delete [] cell;
      delete [] contador;
      delete [] dondeCcell;
    }
    int j=0;
    for(;r<pobSRem;r++,j++){
      if(j==posPARETO)j=0;
      copiaxxx(memoria+r,PARETO+j);
    }
    
  }
}

/*
Metodo que que indica si la poblacion tiene convergencia nominal.
 */
int MicroAG::pobUniforme(){

  Individuo *pp=poblacion;
  for(int i=0;i<M;i+=2){
    if(Domina(pp+i,pp+i+1)!=0)
      return 0;
  }
  return 1;
}


/*
este Metodo manda individuo a individuo, a comparar para ver si es un miembro del frente de pareto.
 */
void MicroAG::guardaPobEnPareto(int mejor1,int mejor2){
  for(int i=0;i<M;i++){
    FunPARETO(poblacion+i);

  }
  /*
  if(restricciones(poblacion[mejor1].genotipo)==0)
    FunPARETO(poblacion+mejor1);
  if(mejor2!=-1)
    if(restricciones(poblacion[mejor2].genotipo)==0)
      FunPARETO(poblacion+mejor2);
  */
}

/*
Metodo que guarda a un individuo en la memoria perteneciente al fretnte de pareto..
 */
void MicroAG::gPARETO(Individuo *ind)
{
  copiaxxx(PARETO+posPARETO,ind);
  posPARETO++;
}

/*
metodo que checa si un individuo domina a almenos uno de ma memoria del frente de pareto, si es asi entonces reemplaza al primer individuo dominado, y elimina a los individuos dominados de la memoria.
 */
int MicroAG::dominaAPareto(Individuo *ind){
  int resultado=0,i=0,j=0,dom=0;
  int *dominados=new(int[posPARETO]);
  int cual=-1;
  while((i < posPARETO)&&(resultado!=-1)){
    resultado = Domina(ind, PARETO+i);
    if(resultado==-1||resultado==0){dom=-1;break;}
    dominados[i]=0;
    if (resultado == 1){
      if(dom==0){
	cual=i;
	if(posPARETO==tamPARETO)
	  hipercubo[PARETO[cual].loc]--;
	copiaxxx(PARETO+cual,ind);
	if(posPARETO==tamPARETO)
	  ind->loc=cualLoc(ind);
	else ind->loc=0;
	if(ind->loc==-1)
	  actualizaLoc();
	else
	  PARETO[cual].loc=ind->loc;
	hipercubo[PARETO[cual].loc]++;
      }
      else dominados[i]=1;
      dom ++;
    }
    i++;
  }
  if(dom>1)
    eliminaDominados(dominados);
  delete dominados;
  return dom;
}
  
/*
Funcion que elimina a los individuos dominados.
 */
void MicroAG::eliminaDominados(int *dominados){
  int i = 0;
  int pospareto=posPARETO;
  int *pareto=new(int[bits*posPARETO]);
  double *objs=new(double[nObjs*posPARETO]);

  for (i = 0; i < posPARETO; i++){
    copia(pareto+(i*bits), PARETO[i].genotipo,bits);
    copia(objs+(i*nObjs),PARETO[i].aptObj,nObjs);
  }
  posPARETO = 0;
  //  inicializaDivisiones();  
  for (i = 0; i < pospareto; i++){
    if (dominados[i]!=1){
      copia(PARETO[posPARETO].genotipo,pareto+(i*bits),bits);
      copia(PARETO[posPARETO].aptObj,objs+(i*nObjs),nObjs);
      PARETO[posPARETO].loc=0;
      posPARETO++;
    }
  }
  delete [] pareto;
  delete [] objs;
}

void MicroAG::copiaxxx(Individuo *ind,Individuo *ind2){
  copia(ind->genotipo,ind2->genotipo,bits);
  copia(ind->aptObj,ind2->aptObj,nObjs);
  ind->loc=ind2->loc;
}  

void  MicroAG::actualizaLoc(){
  double *MAptObj=new(double[nObjs]);
  double *mAptObj=new(double[nObjs]);

  for(int i=0;i<nObjs;i++){
    MAptObj[i]=-NG;
    mAptObj[i]=NG;
  }
  inicializaDivisiones();
  for(int i=0;i<nObjs;i++){
    for(int j=0;j<posPARETO;j++){
      if(mAptObj[i]>PARETO[j].aptObj[i]) mAptObj[i]=PARETO[j].aptObj[i]; //busca la menor aptitud
      if(MAptObj[i]<PARETO[j].aptObj[i]) MAptObj[i]=PARETO[j].aptObj[i]; //busca la mayor aptitud.
    }
    tamDiv[i]=(MAptObj[i]-mAptObj[i])/(nDivs-1); //encuentra el rango para la grid
    inicioMalla[i]=mAptObj[i]-(tamDiv[i]/2);//coloca el punto de partida de la hipercubo
  }
  for(int i=0;i<posPARETO;i++){
    PARETO[i].loc=cualLoc(PARETO+i);
    hipercubo[PARETO[i].loc]++;
  }
  delete [] mAptObj;
  delete [] MAptObj;
}

void MicroAG::inicializaDivisiones(){
  for(long int i=0;i<lMalla;i++)
    hipercubo[i]=0;
}

/*
Asigna una localidad 
 */
long int  MicroAG::cualLoc(Individuo *ind){
  long int temp;
  long int loc=0;
  int i=nObjs-1;
  for(int j=0;j<nObjs;j++,i--){
    temp= (long int)floor((ind->aptObj[j]-inicioMalla[j])/tamDiv[j]);
    if(temp>nDivs||temp<0) return -1;
    loc+=(long int)floor(temp*pow(nDivs,i));
  }
  return loc;
}


void MicroAG::FunPARETO(Individuo *ind){
  int mPoblado;
  int r;
  if(restricciones(ind->genotipo)>0)return;
  if(posPARETO==0){
    gPARETO(ind);
    return;
  }
  if(dominaAPareto(ind)==0){//nadie domina al individuo
    if(posPARETO+1<tamPARETO)
      gPARETO(ind);
    else if(posPARETO+1 == tamPARETO){
      gPARETO(ind);
      actualizaLoc();
    }
    else {
      mPoblado=buscaMasPoblado();
      r=cualLoc(ind);
      if(r==-1){
	int i=buscaIndConLoc(mPoblado);
	if(i==-1) {cout<<"aqui se sale\n";exit(0);}
	copiaxxx(PARETO+i,ind);
	actualizaLoc();
      }
      else if(masPoblado(mPoblado,r)){
	int i=buscaIndConLoc(mPoblado);
	if(i==-1) {
	  cout<<mPoblado<<" "<<i<<" "<<hipercubo[mPoblado]<<"\n";
	  cout<<"aqui se sale2\n";exit(0);
	}
	ind->loc=r;
	copiaxxx(PARETO+i,ind);
	hipercubo[mPoblado]--;
	hipercubo[r]++;
      }
    }
  }
  
} 

int MicroAG::masPoblado(int mPoblado,int r){
  if(mPoblado==r)return 1;
  if(hipercubo[mPoblado]<hipercubo[r]+1)return 0;
  return 1; 
}

int MicroAG::buscaMasPoblado(){
  int mP[2]={-1,-1};
  for(long int i=0;i<lMalla;i++){
    if(mP[1]<hipercubo[i]){
      mP[0]=i;
      mP[1]=hipercubo[i];
    }
  }    
  return mP[0];
}

int MicroAG::buscaIndConLoc(long int mp){
  for(int i=0;i<posPARETO;i++)
    if(PARETO[i].loc==mp)return i;
  return -1;
}
